unit UChartData;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, VCLTee.TeEngine,
  VCLTee.Series, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.TeeComma, Vcl.StdCtrls,

  System.JSON,
  MQTT;

type
  TForm1 = class(TForm)
    TeeCommander1: TTeeCommander;
    Panel1: TPanel;
    Panel2: TPanel;
    Chart1: TChart;
    Series1: TBarSeries;
    Timer1: TTimer;
    Button1: TButton;
    mStatus: TMemo;
    Button2: TButton;
    btnDisconnect: TButton;
    Series2: TBarSeries;
    EditBroker: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    procedure Timer1Timer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Connect;
    procedure GotConnAck(Sender: TObject; ReturnCode: integer);
    procedure btnDisconnectClick(Sender: TObject);
    procedure Chart1AfterDraw(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    procedure Publish;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  MQTT: TMQTT;
  Counter : Integer;

implementation

{$R *.dfm}

procedure TForm1.btnDisconnectClick(Sender: TObject);
begin
  if (Assigned(MQTT)) then
    begin
      MQTT.Disconnect;
      mStatus.Lines.Add('Disconnected');
      FreeAndNil(MQTT);
    end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Timer1.Enabled := not Timer1.Enabled;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Connect;
end;

procedure TForm1.Connect;
begin
  MQTT := TMQTT.Create(EditBroker.Text, 1883);
  MQTT.WillTopic := 'ipc2023';
  MQTT.WillMsg := 'Broker died!';
  // Events
  MQTT.OnConnAck := GotConnAck;
  {MQTT.OnPublish := GotPub;
  MQTT.OnPingResp := GotPingResp;
  MQTT.OnSubAck := GotSubAck;
  MQTT.OnUnSubAck := GotUnSubAck;
  MQTT.OnPubAck := GotPubAck;}

  if MQTT.Connect then
    mStatus.Lines.Add('Connected to ' + 'test.mosquitto.org' + ' on ' + '1883')
  else
    mStatus.Lines.Add('Failed to connect');
end;

procedure TForm1.Chart1AfterDraw(Sender: TObject);
begin
  Publish;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  btnDisconnectClick(sender);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  mStatus.Clear;
  Counter := 0;
  Connect;
end;

procedure TForm1.GotConnAck(Sender: TObject; ReturnCode: integer);
begin
  mStatus.Lines.Add('Connection Acknowledged: ' + IntToStr(ReturnCode));
end;

procedure TForm1.Publish;
Var publishStr, timeStr : String;
    FmtStngs: TFormatSettings;
begin

  if (Series1.Count>0) then
  begin
    //simple test -  json format, two variables.
    publishStr := '[{"id":"IPC1","v":"' +
                  FormatFloat('0.00',Series1.YValues.Last) +
                  '"},{"id":"IPC2","v":"' +
                  FormatFloat('0.00',Series2.YValues.Last) +
                  '"}]';

    FmtStngs.ShortDateFormat := 'yymmdd';
    FmtStngs.TimeSeparator := ':';
    FmtStngs.LongTimeFormat := 'hh:nn:ss';

    timeStr := FormatDateTime( '', Now, FmtStngs );

    if mStatus.Lines.Count > 10 then
      mStatus.Lines.Delete(0);

    if (Assigned(MQTT)) then
      begin
        MQTT.Publish('ipc2023', publishStr);
        mStatus.Lines.Add('Published ' + timeStr);
      end;
  end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Series1.AddXY(Counter,Random(100)/12);
  Series2.AddXY(Counter,Random(100)/15);

  if Series1.Count > 20 then
  Begin
    Series1.Delete(0);
    Series2.Delete(0);
  End;

  inc(Counter);
end;

end.
